﻿using System.Collections.Generic;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;
using System;

namespace Sample
{
    class Program
    {
        static void Main(string[] args)
        {
            string shppath = @"..\..\statetext.shp";
            ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(shppath);
            InMemoryFeatureLayer im = new InMemoryFeatureLayer();
            layer.Open();
            im.Open();
            Collection<Feature> features = layer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            foreach (Feature f in features)
            {
                im.InternalFeatures.Add(f);
            }

            Collection<FeatureSourceColumn> columns = layer.FeatureSource.GetColumns();
            foreach (FeatureSourceColumn col in columns)
            {
                im.Columns.Add(col);
            }

            printtonewshp(im);
        }

        static string shapeName = @"..\..\result.shp";

        // We add all features from InMemoryFeatureLayer to the new featurelayer
        private static void printtonewshp(InMemoryFeatureLayer layer)
        {
            WellKnownType shapeType = WellKnownType.Point;
            ShapeFileType shapeFileType = (ShapeFileType)shapeType;

            layer.Open();

            InMemoryFeatureLayer inMemoryFeatureLayer = (InMemoryFeatureLayer)layer;
            Collection<Feature> allFeatures = inMemoryFeatureLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<FeatureSourceColumn> allColumns = inMemoryFeatureLayer.QueryTools.GetColumns();

            Collection<DbfColumn> dbfColumns = new Collection<DbfColumn>();
            foreach (FeatureSourceColumn featureSourceColumn in allColumns)
            {
                DbfColumn col = GetDbfColumnFromFeatureSourceColumn(featureSourceColumn);
                if (col != null)
                {
                    dbfColumns.Add(col);
                }
            }

            ShapeFileFeatureSource.CreateShapeFile(shapeFileType,
             shapeName, dbfColumns, Encoding.ASCII, OverwriteMode.Overwrite);
            ShapeFileFeatureSource shapeFileLayer = new ShapeFileFeatureSource(
                shapeName, ShapeFileReadWriteMode.ReadWrite);

            shapeFileLayer.Open();

            foreach (Feature feature in allFeatures)
            {
                Dictionary<string, string> shapeColumnValues = new Dictionary<string, string>();
                BaseShape baseShape = feature.GetShape();

                // Loop through all columns and add their values
                foreach (FeatureSourceColumn column in allColumns)
                {
                    string featureColumnValue = feature.ColumnValues[column.ColumnName].ToString();
                    shapeColumnValues.Add(column.ColumnName, featureColumnValue);
                }

                try
                {
                    Feature newf = new Feature(baseShape, shapeColumnValues);
                    shapeFileLayer.BeginTransaction();
                    shapeFileLayer.AddFeature(newf);
                    TransactionResult result = shapeFileLayer.CommitTransaction();
                    Console.WriteLine(result.TransactionResultStatus);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            shapeFileLayer.Close();
            layer.Close();

            Console.Read();
        }

        private static DbfColumn GetDbfColumnFromFeatureSourceColumn(FeatureSourceColumn featureSourceColumn)
        {
            switch (featureSourceColumn.TypeName)
            {
                case "String":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.String, featureSourceColumn.MaxLength, 0);
                case "Memo":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.Memo, featureSourceColumn.MaxLength, 0);
                case "Integer":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.Integer, featureSourceColumn.MaxLength, 0);
                case "Logical":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.Logical, featureSourceColumn.MaxLength, 0);
                case "Double":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.Double, 0, featureSourceColumn.MaxLength);
                case "Date":
                    return new DbfColumn(featureSourceColumn.ColumnName, DbfColumnType.Date, featureSourceColumn.MaxLength, 0);
                default:
                    return null;
            }
        }
    }
}
